@@ -1,7 +1,7 @@
-from math import ceil
-from math import log10
 from os import get_terminal_size
-from os.path import isdir, split
+from os.path import isdir
+from os.path import split
+from re import findall
 from re import sub as re_sub
 from shutil import move
 from typing import Dict
@@ -58,7 +58,7 @@ def check_process(process: str) -> int:
 
 
 def clean_username(username: str, exclude: str = "") -> str:
-    return str(re_sub(rf"[^a-zA-Z0-9\-.~,{exclude}]", "", username.lower().strip()))
+    return str(re_sub(rf"[^a-zA-Z0-9\-.~{exclude}]", "", username.lower().strip()))
 
 
 def clean_string(title: str) -> str:
@@ -100,6 +100,9 @@ def make_journal(id_: Union[int, str], author: str,
     journal.title = title
     journal.date = date
     journal.content = content
+    journal.mentions = sorted(set(filter(bool, map(clean_username, findall(
+        r'<a[^>]*href="(?:(?:https?://)?(?:www.)?furaffinity.net)?/user/([^/">]+)"',
+        content)))))
 
     return journal
 
@@ -108,7 +111,7 @@ def make_submission(id_: Union[int, str], author: str, title: str,
                     date: str, category: str, species: str,
                     gender: str, rating: str, tags: str = "",
                     description: str = "", file_url: str = "",
-                    file_local_url: str = ""
+                    file_local_url: str = "", folder: str = ""
                     ) -> Tuple[Submission, Optional[bytes]]:
     id_ = int(id_)
     assert id_ > 0, "id must be greater than 0"
@@ -123,6 +126,7 @@ def make_submission(id_: Union[int, str], author: str, title: str,
     assert isinstance(description, str), "description must be of type str"
     assert isinstance(file_url, str), "file_url must be of type str"
     assert isinstance(file_local_url, str), "file_local_url must be of type str"
+    assert isinstance(folder, str), "folder must be of type str"
 
     sub: Submission = Submission()
     sub_file: Optional[bytes] = None
@@ -131,13 +135,17 @@ def make_submission(id_: Union[int, str], author: str, title: str,
     sub.title = title
     sub.author = author
     sub.date = date
-    sub.tags = list(filter(bool, map(str.strip, tags.split(","))))
+    sub.tags = sorted(filter(bool, map(str.strip, tags.split(","))))
     sub.category = category
     sub.species = species
     sub.gender = gender
     sub.rating = rating
     sub.description = description
     sub.file_url = file_url
+    sub.folder = folder
+    sub.mentions = sorted(set(filter(bool, map(clean_username, findall(
+        r'<a[^>]*href="(?:(?:https?://)?(?:www.)?furaffinity.net)?/user/([^/">]+)"',
+        description)))))
 
     if file_local_url:
         with open(file_local_url, "rb") as f:
@@ -193,40 +201,12 @@ def print_users(users: List[Dict[str, str]]):
         space_term = get_terminal_size()[0]
     except IOError:
         pass
+
+    space_name_max: int = max([len(u["USERNAME"]) for u in users]) if users else 10
     space_name: int = space_term - (space_folders + 3) - ((space_folder + 3) * 4) - 1
-
-    users_fmt: List[tuple] = [
-        (
-            user["USERNAME"],
-            f.split(",") if (f := user["FOLDERS"]) else 0,
-            (f.count(",") + 1) if (f := user["GALLERY"]) else 0,
-            (f.count(",") + 1) if (f := user["SCRAPS"]) else 0,
-            (f.count(",") + 1) if (f := user["FAVORITES"]) else 0,
-            (f.count(",") + 1) if (f := user["MENTIONS"]) else 0
-        )
-        for user in users
-    ]
-
-    users_fmt.sort(key=lambda usr: usr[0])
-
-    space_name_max: int = max([len(u[0]) for u in users_fmt]) if users_fmt else 10
     space_name = space_name_max if space_name > space_name_max else space_name
-    len_gallery_max: int = int(max([ceil(log10(u[2])) if u[2] else 0 for u in users_fmt])) if users_fmt else 0
-    len_scraps_max: int = int(max([ceil(log10(u[3])) if u[3] else 0 for u in users_fmt])) if users_fmt else 0
-    len_favorites_max: int = int(max([ceil(log10(u[4])) if u[4] else 0 for u in users_fmt])) if users_fmt else 0
-    len_mentions_max: int = int(max([ceil(log10(u[5])) if u[5] else 0 for u in users_fmt])) if users_fmt else 0
 
-    print(
-        f"{'Username':^{space_name}} | {'Folders':^{space_folders}}" +
-        f" | {'Gallery':^{space_folder}} | {'Scraps':^{space_folder}}" +
-        f" | {'Favorites':^{space_folder}} | {'Mentions':^{space_folder}}"
-    )
-    for user, folders, gallery, scraps, favorites, mentions in users_fmt:
-        folders_min: str = ",".join(set(map(lambda f: f[0], folders)))
-        print(
-            f"{user[:space_name]:<{space_name}} | {folders_min:^{space_folders}}" +
-            f" | {f'{gallery:>{len_gallery_max}}':^{space_folder}}" +
-            f" | {f'{scraps:>{len_scraps_max}}':^{space_folder}}" +
-            f" | {f'{favorites:>{len_favorites_max}}':^{space_folder}}" +
-            f" | {f'{mentions:>{len_mentions_max}}':^{space_folder}}"
-        )
+    print(f"{'Username':^{space_name}} | {'Folders':^{space_folders}}")
+    for user in sorted(users, key=lambda usr: usr["USERNAME"].lower()):
+        folders_min: str = ",".join(set(map(lambda f: f[0], user["FOLDERS"])))
+        print(f"{user['USERNAME'][:space_name]:<{space_name}} | {folders_min:^{space_folders}}")
