@@ -34,6 +34,7 @@ from .download import download_submissions as download_submissions_
 from .download import download_users as download_users_
 from .download import download_users_update
 from .download import read_cookies
+from .download import save_journal
 from .download import save_submission
 from .download import write_cookies
 
@@ -97,12 +98,12 @@ def parse_args(args_raw: Iterable[str]) -> Tuple[Dict[str, str], List[str]]:
 
 def check_update(version: str, package: str) -> bool:
     if (latest := latest_version(package)) and latest != version:
-        print(f"New {package} version available: {latest} > {version}")
+        print(f"New {package} version available: {version} > {latest}")
         return True
     return False
 
 
-def help_(comm: str = None, op: str = "", *_rest) -> str:
+def help_(comm: str = "", op: str = "", *_rest) -> str:
     """
     USAGE
         falocalrepo help [<command> [<operation>]]
@@ -150,8 +151,8 @@ def help_(comm: str = None, op: str = "", *_rest) -> str:
 
     if f is None:
         raise UnknownCommand(f"{comm} {op}".strip())
-    else:
-        return cleandoc(f.__doc__)
+
+    return cleandoc(f.__doc__)
 
 
 def init():
@@ -189,21 +190,25 @@ def config_list(db: FADatabase, *_rest):
 def config_cookies(db: FADatabase, *args: str):
     """
     USAGE
-        falocalrepo config cookies [<cookie a> <cookie b>]
+        falocalrepo config cookies [<cookie1 name>=<cookie1 value>] ...
+                    [<cookieN name>=<cookieN value>]
 
     ARGUMENTS
-        <cookie a>  Cookie a from a valid login session
-        <cookie b>  Cookie b from a valid login session
+        <cookie name>   The name of the cookie (e.g. a)
+        <cookie value>  The value of the cookie
 
     DESCRIPTION
         Read or modify stored cookies.
+
+    EXAMPLES
+        falocalrepo config cookies a=a1b2c3d4-1234 b=e5f6g7h8-5678
     """
 
     if not args:
         for c in read_cookies(db):
             print(f"cookie {c['name']}:", c['value'])
     elif len(args) == 2:
-        write_cookies(db, args[0], args[1])
+        write_cookies(db, **parse_args(args)[0])
     else:
         raise MalformedCommand("cookies needs two arguments")
 
@@ -579,6 +584,7 @@ def database_add_submission(db: FADatabase, *args: str):
             * 'species'
             * 'gender'
             * 'rating'
+            * 'folder' gallery or scraps
         The following parameters are optional:
             * 'tags' comma-separated tags
             * 'description'
@@ -592,14 +598,14 @@ def database_add_submission(db: FADatabase, *args: str):
             species='Unspecified / Any' gender=Any rating=General \\
             tags=cat,mouse,cartoon description="$(cat description.html)" \\
             file_url='http://remote.url/to/submission.file' \\
-            file_local_url='path/to/submission.file'
+            file_local_url='path/to/submission.file' folder=gallery
     """
 
     make_params = parameters(args)
     make_params["id_"] = make_params.get("id", "")
     if "id" in make_params:
         del make_params["id"]
-    save_submission(db, *make_submission(**make_params))
+    save_submission(db, *make_submission(**make_params), user_update=False)
 
 
 def database_add_journal(db: FADatabase, *args: str):
@@ -630,8 +636,7 @@ def database_add_journal(db: FADatabase, *args: str):
     make_params["id_"] = make_params.get("id", "")
     if "id" in make_params:
         del make_params["id"]
-    db.journals.save_journal(dict(make_journal(**make_params)))
-    db.commit()
+    save_journal(db, make_journal(**make_params), user_update=False)
 
 
 def database_remove_users(db: FADatabase, *args: str):
@@ -667,8 +672,6 @@ def database_remove_submissions(db: FADatabase, *args: str):
     for sub in args:
         print("Deleting", sub)
         del db.submissions[int(sub)]
-        for (user, *_) in db.users.find_from_submission(int(sub)):
-            db.users.remove_submission(user, int(sub))
         db.commit()
 
 
@@ -687,8 +690,6 @@ def database_remove_journals(db: FADatabase, *args: str):
     for jrn in args:
         print("Deleting", jrn)
         del db.journals[int(jrn)]
-        for (user, *_) in db.users.find_from_journal(int(jrn)):
-            db.users.remove_journal(user, int(jrn))
     db.commit()
 
 
@@ -883,6 +884,9 @@ def console(comm: str = "", *args: str) -> None:
     elif check_process(p := "falocalrepo") > 1:
         raise MultipleInstances(f"Another instance of {p} was detected")
 
+    if environ.get("FALOCALREPO_DEBUG", None) is not None:
+        print(f"Using FALOCALREPO_DEBUG")
+
     # Initialise and prepare database
     database_path = "FA.db"
 
